<?php
/**
 * osCommerce Online Merchant
 *
 * @copyright (c) 2020 osCommerce; https://www.oscommerce.com
 * @license MIT; https://www.oscommerce.com/license/mit.txt
 *
 * @author Valeriy Rudenko <ruden@osclab.com>
 * @copyright (c) 2020 OSCLab: https://osclab.com
 */

  class bm_recaptcha {
    public $code;
    public $group;
    public $title;
    public $description;
    public $sort_order;
    public $enabled = false;
    public $verify_url;

    public function __construct() {
      $this->code = get_class($this);
      $this->group = basename(dirname(__FILE__));

      $this->title = MODULE_BOXES_RECAPTCHA_TITLE;
      $this->description = MODULE_BOXES_RECAPTCHA_DESCRIPTION;

      if ($this->check()) {
        $this->sort_order = MODULE_BOXES_RECAPTCHA_SORT_ORDER;
        $this->enabled = (MODULE_BOXES_RECAPTCHA_STATUS == 'True');
      }

      $this->verify_url = 'https://www.google.com/recaptcha/api/siteverify';
    }

    public function execute($form_name = null, $param_action_value = null) {
      global $oscTemplate, $sessiontoken;

      if (empty($form_name) || empty($param_action_value)) {
        return null;
      }

      if ($this->isAction($param_action_value) && $this->isFormId($sessiontoken)) {
        $error = true;

        if (isset($_POST[$sessiontoken])) {
          $params = http_build_query(array('secret' => htmlspecialchars(trim(MODULE_BOXES_RECAPTCHA_SECRET_KEY)),
                                           'response' => tep_db_prepare_input($_POST[$sessiontoken])), '', '&');

          if (extension_loaded('curl')) {
            $response = $this->queryCurlPost($params);
          } else {
            $response = $this->queryPost(array('http' => array('header' => "Content-type: application/x-www-form-urlencoded\r\n",
                                                               'method' => 'POST',
                                                               'content' => $params)));
          }

          $response = json_decode($response, true);


          if ($response['success'] == true && $response['action'] == hash('crc32', $sessiontoken . MODULE_BOXES_RECAPTCHA_SECRET_KEY) && floatval($response['score']) >= floatval(htmlspecialchars(MODULE_BOXES_RECAPTCHA_SCORE))) {
            $error = false;
          }
        }

        if ($error) {
          tep_session_unregister('sessiontoken');
          tep_redirect(tep_href_link('index.php'));
        }
      }

      if (class_exists('oscTemplate') && !empty($oscTemplate)) {
        $oscTemplate->addBlock($this->renderJavascript($form_name, $sessiontoken), 'footer_scripts');
      }
    }

    public function isEnabled() {
      return $this->enabled;
    }

    public function check() {
      return defined('MODULE_BOXES_RECAPTCHA_STATUS');
    }

    public function install() {
      tep_db_query("INSERT INTO configuration (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) VALUES ('Enable Google reCAPTCHA v3 Module', 'MODULE_BOXES_RECAPTCHA_STATUS', 'True', 'Do you want to enable the Google reCAPTCHA v3 module?', '6', '0', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
      tep_db_query("INSERT INTO configuration (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) VALUES ('Site key', 'MODULE_BOXES_RECAPTCHA_SITE_KEY', '', 'Enter the site key.', '6', '0', now())");
      tep_db_query("INSERT INTO configuration (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) VALUES ('Secret key', 'MODULE_BOXES_RECAPTCHA_SECRET_KEY', '', 'Enter the secret key.', '6', '0', now())");
      tep_db_query("INSERT INTO configuration (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) VALUES ('Threshold score', 'MODULE_BOXES_RECAPTCHA_SCORE', '0.5', 'Recaptcha returns a score from 0.0 to 1.0 (1.0 is very likely a good interaction, 0.0 is very likely a bot).', '6', '0', 'tep_cfg_select_option(array(\'0.0\', \'0.1\', \'0.2\', \'0.3\', \'0.4\', \'0.5\', \'0.6\', \'0.7\', \'0.8\', \'0.9\',  \'1.0\'), ', now())");
      tep_db_query("INSERT INTO configuration (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) VALUES ('Sort Order', 'MODULE_BOXES_RECAPTCHA_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '0', now())");
    }

    public function remove() {
      tep_db_query("delete from configuration where configuration_key in ('" . implode("', '", $this->keys()) . "')");
    }

    public function keys() {
      return array('MODULE_BOXES_RECAPTCHA_STATUS', 'MODULE_BOXES_RECAPTCHA_SITE_KEY', 'MODULE_BOXES_RECAPTCHA_SECRET_KEY', 'MODULE_BOXES_RECAPTCHA_SCORE', 'MODULE_BOXES_RECAPTCHA_SORT_ORDER');
    }

    public function isAction($value) {
      return isset($_REQUEST['action']) && $_REQUEST['action'] == $value;
    }

    public function isFormId($sessiontoken) {
      return isset($_POST['formid']) && $_POST['formid'] == $sessiontoken;
    }

    public function queryCurlPost($params) {
      $curl = curl_init($this->verify_url);
      curl_setopt_array($curl, array(CURLOPT_POST => true,
                                     CURLOPT_POSTFIELDS => $params,
                                     CURLOPT_HTTPHEADER => array('Content-Type: application/x-www-form-urlencoded'),
                                     CURLINFO_HEADER_OUT => false,
                                     CURLOPT_HEADER => false,
                                     CURLOPT_RETURNTRANSFER => true,
                                     CURLOPT_SSL_VERIFYPEER => true));
      $response = curl_exec($curl);
      curl_close($curl);

      return $response;
    }

    public function queryPost(array $params) {
      $context = stream_context_create($params);

      return file_get_contents($this->verify_url, false, $context);
    }

    public function renderJavascript($forms, $name) {
      $site_key = htmlspecialchars(trim(MODULE_BOXES_RECAPTCHA_SITE_KEY));
      $action = hash('crc32', $name . MODULE_BOXES_RECAPTCHA_SECRET_KEY);

      return <<<JS
<script>
  function recaptchaCallback() {
    const form = document.forms['{$forms}'];
      
    if (form) {
      const i = document.createElement('input');
        
      i.type = 'hidden';
      i.name = '{$name}';
          
      form.appendChild(i);
       
      grecaptcha.execute('{$site_key}', {action: '{$action}'}).then(function(token) {
        i.value = token;
      });
    }
  }
</script>
<script src="https://www.google.com/recaptcha/api.js?onload=recaptchaCallback&render={$site_key}" async defer></script>
JS;
    }
  }